/*
 * Copyright 2005-2010 the original author or authors.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wamblee.inject;

/**
 * This abstract base class performs injection at construction. Be sure not to
 * initialize fields of derived classes to null as these will override the
 * initializations of this base class.
 * 
 * This class uses {@link InjectorBuilder} to obtain an implementation of a
 * {@link InjectorFactory} to use.
 * 
 * @author Erik Brakkee
 */
public abstract class Injectable {

    private final Injector injector = InjectorBuilder.getInjector();

    /**
     * Inheritance style constructor.
     */
    protected Injectable() {
        inject();
    }

    /**
     * Inject references. This can be useful when injection is to be done again
     * after construction to obtain new references or after zeroing out of the
     * references.
     */
    public void inject() {
        injector.inject(this);
    }

    /**
     * Gets the default injector.
     * 
     * @return Injector.
     */
    public Injector getInjector() {
        return injector;
    }
}
